@@ -60,7 +60,9 @@ struct sockaddr_in {
 struct sockaddr_in6 {
     sa_family_t         sin6_family;
     in_port_t           sin6_port;
+    uint32_t            sin6_flowinfo;
     struct in6_addr     sin6_addr;
+    uint32_t            sin6_scope_id;
     ...;
 };
 
@@ -68,9 +70,14 @@ struct sockaddr_in6 {
 #define INET6_ADDRSTRLEN ...
 
 #define C_IN ...
+#define C_CHAOS ...
+#define C_HS ...
+#define C_NONE ...
+#define C_ANY ...
 #define T_A  ...
 #define T_AAAA  ...
 #define T_ANY  ...
+#define T_CAA ...
 #define T_CNAME ...
 #define T_MX  ...
 #define T_NAPTR ...
@@ -109,6 +116,7 @@ typedef int... ares_socklen_t;
 #define ARES_ELOADIPHLPAPI           ...
 #define ARES_EADDRGETNETWORKPARAMS   ...
 #define ARES_ECANCELLED         ...
+#define ARES_ESERVICE           ...
 
 #define ARES_FLAG_USEVC         ...
 #define ARES_FLAG_PRIMARY       ...
@@ -201,6 +209,11 @@ typedef int  (*ares_sock_create_callback)(ares_socket_t socket_fd,
                                           int type,
                                           void *data);
 
+typedef void (*ares_addrinfo_callback)(void *arg,
+                                   int status,
+                                   int timeouts,
+                                   struct ares_addrinfo *res);
+
 struct ares_channeldata;
 typedef struct ares_channeldata *ares_channel;
 
@@ -240,27 +253,33 @@ struct ares_addr6ttl {
   int             ttl;
 };
 
+struct ares_caa_reply {
+  struct ares_caa_reply  *next;
+  int                     critical;
+  unsigned char          *property;
+  size_t                  plength;
+  unsigned char          *value;
+  size_t                  length;
+};
+
 struct ares_srv_reply {
   struct ares_srv_reply  *next;
   char                   *host;
   unsigned short          priority;
   unsigned short          weight;
   unsigned short          port;
-  int                     ttl;
 };
 
 struct ares_mx_reply {
   struct ares_mx_reply   *next;
   char                   *host;
   unsigned short          priority;
-  int                     ttl;
 };
 
 struct ares_txt_reply {
   struct ares_txt_reply  *next;
   unsigned char          *txt;
   size_t                  length;
-  int                     ttl;
 };
 
 struct ares_txt_ext {
@@ -268,7 +287,6 @@ struct ares_txt_ext {
   unsigned char            *txt;
   size_t                   length;
   unsigned char            record_start;
-  int                      ttl;
 };
 
 struct ares_naptr_reply {
@@ -279,7 +297,6 @@ struct ares_naptr_reply {
   char                    *replacement;
   unsigned short           order;
   unsigned short           preference;
-  int                      ttl;
 };
 
 struct ares_soa_reply {
@@ -290,8 +307,46 @@ struct ares_soa_reply {
   unsigned int retry;
   unsigned int expire;
   unsigned int minttl;
-  int          ttl;
 };
+/*
+ * Similar to addrinfo, but with extra ttl and missing canonname.
+ */
+struct ares_addrinfo_node {
+  int                        ai_ttl;
+  int                        ai_flags;
+  int                        ai_family;
+  int                        ai_socktype;
+  int                        ai_protocol;
+  ares_socklen_t             ai_addrlen;
+  struct sockaddr           *ai_addr;
+  struct ares_addrinfo_node *ai_next;
+};
+
+/*
+ * alias - label of the resource record.
+ * name - value (canonical name) of the resource record.
+ * See RFC2181 10.1.1. CNAME terminology.
+ */
+struct ares_addrinfo_cname {
+  int                         ttl;
+  char                       *alias;
+  char                       *name;
+  struct ares_addrinfo_cname *next;
+};
+
+struct ares_addrinfo {
+  struct ares_addrinfo_cname *cnames;
+  struct ares_addrinfo_node  *nodes;
+  ...;
+};
+
+struct ares_addrinfo_hints {
+  int ai_flags;
+  int ai_family;
+  int ai_socktype;
+  int ai_protocol;
+};
+
 
 struct ares_addr_node {
   struct ares_addr_node *next;
@@ -341,6 +396,15 @@ void ares_set_socket_callback(ares_channel channel,
                                            ares_sock_create_callback callback,
                                            void *user_data);
 
+void ares_getaddrinfo(ares_channel channel,
+                                   const char* node,
+                                   const char* service,
+                                   const struct ares_addrinfo_hints* hints,
+                                   ares_addrinfo_callback callback,
+                                   void* arg);
+
+void ares_freeaddrinfo(struct ares_addrinfo* ai);
+
 void ares_send(ares_channel channel,
                             const unsigned char *qbuf,
                             int qlen,
@@ -439,13 +503,16 @@ int ares_parse_aaaa_reply(const unsigned char *abuf,
                                        struct ares_addr6ttl *addrttls,
                                        int *naddrttls);
 
+int ares_parse_caa_reply(const unsigned char* abuf,
+                         int alen,
+                         struct ares_caa_reply** caa_out);
+
 int ares_parse_ptr_reply(const unsigned char *abuf,
                                       int alen,
                                       const void *addr,
                                       int addrlen,
                                       int family,
-                                      struct hostent **host,
-                                      int *hostttl);
+                                      struct hostent **host);
 
 int ares_parse_ns_reply(const unsigned char *abuf,
                                      int alen,
@@ -489,7 +556,34 @@ const char *ares_inet_ntop(int af, const void *src, char *dst,
                                         ares_socklen_t size);
 
 int ares_inet_pton(int af, const char *src, void *dst);
+"""
 
+CALLBACKS = """
+extern "Python" void _sock_state_cb(void *data,
+                                    ares_socket_t socket_fd,
+                                    int readable,
+                                    int writable);
+
+extern "Python" void _host_cb(void *arg,
+                              int status,
+                              int timeouts,
+                              struct hostent *hostent);
+
+extern "Python" void _nameinfo_cb(void *arg,
+                                  int status,
+                                  int timeouts,
+                                  char *node,
+                                  char *service);
+
+extern "Python" void _query_cb(void *arg,
+                               int status,
+                               int timeouts,
+                               unsigned char *abuf,
+                               int alen);
+extern "Python" void _addrinfo_cb(void *arg,
+                                  int status,
+                                  int timeouts,
+                                  struct ares_addrinfo *res);
 """
 
 INCLUDES = """
@@ -502,13 +596,11 @@ INCLUDES = """
 # include <netdb.h> /* struct hostent */
 # include <netinet/in.h> /* struct sockaddr_in/sockaddr_in6 */
 #endif
-#include <nameser.h>
-#define CARES_STATICLIB 1 /* static link it */
+#include <ares_nameser.h>
 #include <ares.h>
 """
 
 
 ffi = cffi.FFI()
-ffi.cdef(PLATFORM_TYPES + TYPES + FUNCTIONS)
+ffi.cdef(PLATFORM_TYPES + TYPES + FUNCTIONS + CALLBACKS)
 ffi.set_source('_cares', INCLUDES)
-
